{
   MegaBrutal's SMTP Server (MgSMTP)
   Copyright (C) 2010-2012 MegaBrutal

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Affero General Public License for more details.

   You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
}


{$MODE DELPHI}
unit Bounce;

interface
uses SysUtils, Classes, Common;

const

   DSMSG_CONNECTIONFAIL    = 'Failed to establish connection with relay server: ';
   DSMSG_INTERNALFAIL      = 'Internal failure: ';
   {DSMSG_MAILBOXNOTEXISTS  = 'Although the recipient addressed a valid local mailbox'#13#10
                           + 'at the time when I queued it, meanwhile it became invalid'#13#10
                           + 'due to changes in the mail server''s configuration, thanks'#13#10
                           + 'to the beloved postmaster. I apologise.';}
   DSMSG_MAILBOXNOTEXISTS  = 'Local mailbox does not exist.';
   DSMSG_QUOTAEXCEEDED     = 'User quota exceeded.';


   function GenerateBounceMessage(FailedRecipient: TRecipient; Headers: TStrings; ReturnPath: string): TStrings; overload;
   function GenerateBounceMessage(Envelope: TEnvelope; Headers: TStrings): TStrings; overload;


implementation


function GetFailureTypeStr(Status: integer): string;
begin
   if (Status and DS_UNEXPECTEDFAIL) <> 0 then
      Result:= 'an unexpected'
   else if (Status and DS_CONNECTIONFAIL) <> 0 then
      Result:= 'a connection'
   else if (Status and DS_INTERNALFAIL) <> 0 then
      Result:= 'an internal'
   else if (Status and DS_PERMANENT) <> 0 then
      Result:= 'a permanent'
   else if (Status and DS_DELAYED) <> 0 then
      Result:= 'a temporary'
   else
      Result:= '';
end;

procedure GenerateHeader(Msg: TStrings; ReturnPath: string);
begin
   with Msg do begin
      Add('From: Mail Delivery System <MAILER-DAEMON@' + MainServerConfig.Name + '>');
      Add('To: <' + ReturnPath + '>');
      Add('Subject: Delivery Status Notification');
      Add('');
      Add('This is the mail delivery system at host ' + MainServerConfig.Name + ',');
      Add('embodied by MgSMTP software version ' + MainServerConfig.VersionStr + '.');
      Add('');
   end;
end;

procedure AddTechnicalDetails(Msg: TStrings; FailedRecipient: TRecipient);
begin
   with Msg do begin
      if (FailedRecipient.Data and DS_SMTPFAIL) <> 0 then begin
         Add('The targetted mail server has rejected the message:');
         Add(IntToStr(FailedRecipient.Data and DS_SMTPREPLYMASK) + #32 + CleanEOLN(FailedRecipient.RMsg));
      end
      else begin
         if Length(FailedRecipient.RMsg) > 0 then
            Add(CleanEOLN(FailedRecipient.RMsg))
         else begin
            Add('No error message. This is an unexpected failure.');
            Add('Possible that the relay server has unexpectedly');
            Add('closed the connection.');
         end;
      end;
   end;
end;

procedure AddHeaders(Msg, Headers: TStrings);
begin
   with Msg do begin
      Add('');
      Add('Below you can see the headers of your undelivered message.');
      Add('Pay attention to the "Subject" and "Message-Id" headers to');
      Add('get a clue which was it exactly.');
      Add('------------------------------------------------------------');
      AddStrings(Headers);
   end;
end;

function GenerateBounceMessage(FailedRecipient: TRecipient; Headers: TStrings; ReturnPath: string): TStrings;
var Msg: TStrings;
begin
   Msg:= TStringList.Create;
   with Msg do begin
      GenerateHeader(Msg, ReturnPath);
      Add('I am sorry to inform you that I encountered a problem');
      Add('while I was trying to deliver your message to the following');
      Add('recipient: <' + FailedRecipient.Address + '>.');
      Add('This is ' + GetFailureTypeStr(FailedRecipient.Data) + ' failure.');
      Add('');
      Add('Technical details:');
      AddTechnicalDetails(Msg, FailedRecipient);
      if (FailedRecipient.Data and DS_DELAYED) <> 0 then begin
         Add('');
         Add('It seems it''s a temporary failure, so I''ll keep on trying.');
      end
      else if (FailedRecipient.Data and DS_PERMANENT) <> 0 then begin
         Add('');
         Add('It''s a permanent failure, I''ve given up trying.');
      end;
      AddHeaders(Msg, Headers);
   end;
   Result:= Msg;
end;

function GenerateBounceMessage(Envelope: TEnvelope; Headers: TStrings): TStrings;
var Msg: TStrings; FailedRecipient: TRecipient; i: integer;
begin
   Msg:= TStringList.Create;
   with Msg do begin
      GenerateHeader(Msg, Envelope.ReturnPath);
      Add('I am sorry to inform you that I encountered several');
      Add('problems while I was trying to deliver your message to');
      Add('multiple recipients.');
      Add('');
      Add('Details follow:');
      Add('');
      for i:= 0 to Envelope.GetNumberOfRecipients - 1 do begin
         Add('');
         FailedRecipient:= Envelope.GetRecipient(i);
         Add('Recipient <' + FailedRecipient.Address + '>:');
         AddTechnicalDetails(Msg, FailedRecipient);
         Add('');
         Add('This is ' + GetFailureTypeStr(FailedRecipient.Data) + ' failure.');
         if (FailedRecipient.Data and DS_DELAYED) <> 0 then
            Add('Delivery will be retried by certain intervals.')
         else if (FailedRecipient.Data and DS_PERMANENT) <> 0 then
            Add('Delivery to this recipient has failed permanently.');
         Add('');
      end;
      AddHeaders(Msg, Headers);
   end;
   Result:= Msg;
end;


end.
